package go.meethour.io.flutter.sdk

import android.app.KeyguardManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.res.Configuration
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.WindowManager
import go.meethour.io.flutter.sdk.MeetHourPlugin.Companion.MEETHOUR_MEETING_CLOSE
import go.meethour.io.flutter.sdk.MeetHourPlugin.Companion.MEETHOUR_PLUGIN_TAG
import go.meethour.io.MeetHourSDK.android.MeetHourActivity
import go.meethour.io.MeetHourSDK.android.MeetHourConferenceOptions

/**
 * Activity extending MeetHourActivity in order to override the conference events
 */
class MeetHourPluginActivity : MeetHourActivity() {
    companion object {
        @JvmStatic
        fun launchActivity(context: Context?,
                           options: MeetHourConferenceOptions) {
            var intent = Intent(context, MeetHourPluginActivity::class.java).apply {
                action = "go.meethour.io.CONFERENCE"
                putExtra("MeetHourConferenceOptions", options)
            }
            context?.startActivity(intent)
        }
    }

    var onStopCalled: Boolean = false;

//    override fun onPictureInPictureModeChanged(isInPictureInPictureMode: Boolean, newConfig: Configuration?) {
//        super.onPictureInPictureModeChanged(isInPictureInPictureMode, newConfig)
//
//        if (isInPictureInPictureMode){
//            MeetHourEventStreamHandler.instance.onPictureInPictureWillEnter()
//        }
//        else {
//            MeetHourEventStreamHandler.instance.onPictureInPictureTerminated()
//        }
//
//        if (isInPictureInPictureMode == false && onStopCalled) {
//            // Picture-in-Picture mode has been closed, we can (should !) end the call
//            getMeetHourView().leave()
//        }
//    }

    private val myReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            when (intent?.action) {
                MEETHOUR_MEETING_CLOSE -> finish()
            }
        }
    }

    override fun onStop() {
        super.onStop()
        onStopCalled = true;
        unregisterReceiver(myReceiver)
    }

    override fun onResume() {
        super.onResume()
        onStopCalled = false
        registerReceiver(myReceiver, IntentFilter(MEETHOUR_MEETING_CLOSE))
    }

    override fun onConferenceWillJoin(data: HashMap<String, Any>) {
        Log.d(MEETHOUR_PLUGIN_TAG, String.format("MeetHourPluginActivity.onConferenceWillJoin: %s", data))
        MeetHourEventStreamHandler.instance.onConferenceWillJoin(data)
        super.onConferenceWillJoin(data)
    }

    override fun onConferenceJoined(data: HashMap<String, Any>) {
        Log.d(MEETHOUR_PLUGIN_TAG, String.format("MeetHourPluginActivity.onConferenceJoined: %s", data))
        MeetHourEventStreamHandler.instance.onConferenceJoined(data)
        super.onConferenceJoined(data)
    }

    override fun onConferenceTerminated(data: HashMap<String, Any>) {

        Log.d(MEETHOUR_PLUGIN_TAG, String.format("MeetHourPluginActivity.onConferenceTerminated: %s", data))
        MeetHourEventStreamHandler.instance.onConferenceTerminated(data)
        super.onConferenceTerminated(data)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        turnScreenOnAndKeyguardOff();
    }

    override fun onDestroy() {
        super.onDestroy()
        turnScreenOffAndKeyguardOn();
    }

    private fun turnScreenOnAndKeyguardOff() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
            // For newer than Android Oreo: call setShowWhenLocked, setTurnScreenOn
            setShowWhenLocked(true)
            setTurnScreenOn(true)

            // If you want to display the keyguard to prompt the user to unlock the phone:
            val keyguardManager = getSystemService(Context.KEYGUARD_SERVICE) as KeyguardManager
            keyguardManager?.requestDismissKeyguard(this, null)
        } else {
            // For older versions, do it as you did before.
            window.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED
                    or WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
                    or WindowManager.LayoutParams.FLAG_FULLSCREEN
                    or WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    or WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    or WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON)
        }
    }

    private fun turnScreenOffAndKeyguardOn() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
            setShowWhenLocked(false)
            setTurnScreenOn(false)
        } else {
            window.clearFlags(
                    WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED
                            or WindowManager.LayoutParams.FLAG_FULLSCREEN
                            or WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
                            or WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                            or WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                            or WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON
            )
        }
    }
}
